{$I DacDemo.inc}
unit LiteDacDemoForm;

interface

uses
  SysUtils, Classes, DB, Variants,
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, Forms,
  Dialogs, Menus, ImgList, StdCtrls, ComCtrls, Buttons, ExtCtrls, ToolWin,
  LiteDacAbout, DemoFrame, DemoForm, DemoBase, DBAccess, DAScript,
  LiteScript, LiteAccess, LiteDacVcl, LiteCall, LiteConsts;

type
  TLiteDacForm = class(TDemoForm)
    LiteConnection: TLiteConnection;
    LiteConnectDialog1: TLiteConnectDialog;
    scCreate: TLiteScript;
    scDrop: TLiteScript;
    cbDirect: TCheckBox;
    procedure lbAboutClick(Sender: TObject); override;
    procedure cbDebugClick(Sender: TObject);
    procedure cbDirectClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure sbConnectClick(Sender: TObject);
  private
  protected
    //Product customization
    function GetConnection: TCustomDAConnection; override;
    function ApplicationTitle: string; override;
    function ProductName: string; override;
    procedure RegisterDemos; override;

    procedure InitializeDemoFrame(Frame: TDemoFrame; DemoType: TDemoType); override;
  public
    function ProductColor: TColor; override;
    procedure ExecCreateScript; override;
    procedure ExecDropScript; override;
  end;

var
  LiteDacForm: TLiteDacForm;

implementation

uses
{$IFDEF CRDBGRID}
{$IFNDEF FPC}
  CRDBGrid,
{$ENDIF}
{$ENDIF}
{$IFNDEF FPC}
  Pictures,
{$ENDIF}
{$IFNDEF STD}
  Dump, Loader,
{$ENDIF}
  CachedUpdates, ConnectDialog, MasterDetail, Query, Sql, 
  FilterAndIndex, Table, Text, UpdateSQL, VTable, Lock, Functions,
  AggregateAndCalc;

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

function TLiteDacForm.GetConnection: TCustomDAConnection;
begin
  Result := LiteConnection;
end;

function TLiteDacForm.ProductColor: TColor;
begin
  Result := $006E5200;
end;

procedure TLiteDacForm.ExecCreateScript;
begin
  scCreate.OnError := OnScriptError;
  scCreate.Execute;
end;

procedure TLiteDacForm.ExecDropScript;
begin
  scDrop.OnError := OnScriptError;
  scDrop.Execute;
end;

function TLiteDacForm.ApplicationTitle: string;
begin
  Result := 'SQLite Data Access Components demos';
end;

function TLiteDacForm.ProductName: string;
begin
  Result := 'LiteDAC';
end;

procedure TLiteDacForm.RegisterDemos;
begin
  Demos.RegisterCategory('LiteDAC Demo', 'LiteDAC Demo');
  Demos.RegisterCategory('Working with components', 'Working with components');
  Demos.RegisterCategory('General demos', 'General demos');
  Demos.RegisterCategory('SQLite-specific', 'SQLite-specific');

  Demos.RegisterDemo('ConnectDialog', 'Customizing login dialog',
    'Demonstrates how to customize the LiteDAC connect dialog. Changes the ' +
    'standard LiteDAC connect dialog to the custom connect dialog. ' +
    'The customized sample dialog is inherited from the TForm class.',
    'Working with components', TConnectDialogFrame, 2);
{$IFDEF CRDBGRID}
{$IFNDEF FPC}
  Demos.RegisterDemo('CRDBGrid', 'Using TCRDBGrid component',
    'Demonstrates how to work with the TCRDBGrid component.  Shows off ' +
    'the main TCRDBGrid features, like filtering, searching, stretching, ' +
    'using compound headers, and more.',
    'Working with components', TCRDBGridFrame, 3);
{$ENDIF}
{$ENDIF}
{$IFNDEF STD}
  Demos.RegisterDemo('Dump', 'Using TLiteDump component',
    'Demonstrates how to backup data from tables with the TLiteDump component. ' +
    'Shows how to use scripts created during back up to restore table data. ' +
    'This demo lets you back up a table either by specifying the table name ' +
    'or by writing a SELECT query.',
    'Working with components', TDumpFrame, 4);
  Demos.RegisterDemo('Loader', 'Using TLiteLoader component',
    'Uses the TLiteLoader component to quickly load data into a server table. ' +
    'This demo also compares the two TLiteLoader data loading handlers: ' +
    'GetColumnData and PutData.',
    'Working with components', TLoaderFrame, 5);
{$ENDIF}
  Demos.RegisterDemo('Query', 'Using TLiteQuery component',
    'Demonstrates working with TLiteQuery, which is one of the most useful ' +
    'LiteDAC components. Includes many LiteQuery usage scenarios. Demonstrates ' +
    'how to execute queries and how to edit data and export it to XML files. ' +
    'Note: This is a very good introductory demo. We recommend starting here ' +
    'when first becoming familiar with LiteDAC.',
    'Working with components', TQueryFrame, 6);
  Demos.RegisterDemo('Sql', 'Using TLiteSQL component',
    'Uses the TLiteSQL component to execute SQL statements. Demonstrates how ' +
    'to work in a separate thread and how to break long-duration query execution.',
    'Working with components', TSqlFrame, 7);
  Demos.RegisterDemo('Table', 'Using TLiteTable component',
    'Demonstrates how to use TLiteTable to work with data from a single table ' +
    'without manually writing any SQL queries. Performs data sorting and ' +
    'filtering and retrieves results for browsing and editing.',
    'Working with components', TTableFrame, 9);
  Demos.RegisterDemo('UpdateSQL', 'Using TLiteUpdateSQL component',
    'Demonstrates using the TLiteUpdateSQL component to customize update commands. ' +
    'Lets you optionally use TLiteSQL and TLiteQuery objects for carrying ' +
    'out insert, delete, query, and update commands.',
    'Working with components', TUpdateSQLFrame, 10);
  Demos.RegisterDemo('VirtualTable', 'Using TVirtualTable component',
    'Demonstrates working with the TVirtualTable component. This sample shows ' +
    'how to fill virtual dataset with data from other datasets, filter data ' +
    'by a given criteria, locate specified records, perform file operations, ' +
    'and change data and table structure.',
    'Working with components', TVirtualTableFrame, 11, 'VTable');

  Demos.RegisterDemo('CachedUpdates', 'Cached updates, transaction control',
    'Demonstrates how to perform the most important tasks of working with data ' +
    'in CachedUpdates mode, including highlighting uncommitted changes, ' +
    'managing transactions, and committing changes in a batch.',
    'General demos', TCachedUpdatesFrame, 1);
  Demos.RegisterDemo('FilterAndIndex', 'Using Filter and IndexFieldNames',
    'Demonstrates LiteDAC''s local storage functionality. This sample shows ' +
    'how to perform local filtering, sorting and locating by multiple fields, ' +
    'including by calculated and lookup fields.',
    'General demos', TFilterAndIndexFrame, 1);
{$IFNDEF FPC}
  Demos.RegisterDemo('AggregateAndCalc', 'Using Calculated, InternalCalc and Aggregate Fields',
    'Shows how to work with Calculated, Internal Calculated and Aggregate fields.',
    'General demos', TAggregateFrame, 1);
{$ENDIF}
  Demos.RegisterDemo('MasterDetail', 'Master/detail relationship',
    'Uses LiteDAC functionality to work with master/detail relationships. ' +
    'This sample shows how to use local master/detail functionality. ' +
    'Demonstrates different kinds of master/detail linking, including linking ' +
    'by SQL, by simple fields, and by calculated fields.',
    'General demos', TMasterDetailFrame, 1);
  Demos.RegisterDemo('Lock', 'Editing data with row-level locking',
    'Demonstrates two kinds of row-level locking (Update locking and Exclusive ' +
    'locking). This functionality is based on locking hints.',
    'General demos', TLockFrame, 1);
{$IFNDEF FPC}
  Demos.RegisterDemo('Pictures', 'Working with BLOB field',
    'Uses LiteDAC functionality to work with graphics. The sample demonstrates ' +
    'how to retrieve binary data from SQLite database and display it on ' +
    'visual components. Sample also shows how to load and save pictures ' +
    'to files and to the database.',
    'SQLite-specific', TPicturesFrame, 1);
{$ENDIF}
  Demos.RegisterDemo('Text', 'Working with long string fields',
    'Uses LiteDAC functionality to work with text. The sample demonstrates ' +
    'how to retrieve text data from SQLite database and display it on visual ' +
    'components. Sample also shows how to load and save text to files and to the database.',
    'SQLite-specific', TTextFrame, 1);
{$IFNDEF STD}
  Demos.RegisterDemo('Functions', 'Working with user-defined functions',
    'Uses LiteDAC functionality to work with SQLite functions. The sample ' +
    'demonstrates how to define custom functions for future use in ' +
    'SQL-statements, how to overload user-defined functions and how to ' +
    'override built-in SQLite functions.',
    'SQLite-specific', TFunctionsFrame, 1);
{$ENDIF}

  Demos.RegisterCategory('Miscellaneous', '', -1, True);
{$IFDEF CLR}
  Demos.RegisterDemo('AspNet', 'AspNet',
    'Uses LiteDataAdapter to create a simple ASP .NET application. This demo ' +
    'shows how to create an ASP.NET application that lets you connect to a ' +
    'database and execute queries. Application displays query results in a ' +
    'DataGrid and sends user changes back to the database.',
    'Miscellaneous', nil, 1, '', True);
  Demos.RegisterDemo('WinForms', 'AspNet',
    'Shows how to use LiteDAC to create a WinForm application. This demo project ' +
    'creates a simple WinForms application and fills a data grid from an ' +
    'LiteDataAdapter data source.',
    'Miscellaneous', nil, 1, '', True);
{$ENDIF}
{$IFNDEF CLR}
  Demos.RegisterDemo('Dll', 'Dll',
    'Demonstrates creating and loading DLLs for LiteDAC-based projects. ' +
    'This demo project consists of two parts - an LiteDll project that creates ' +
    'a DLL of a form that sends a query to the server and displays its results, ' +
    'and an LiteExe project that can be executed to display a form for loading ' +
    'and running this DLL. Allows you to build a dll for one LiteDAC-based ' +
    'project and load and test it from a separate application.',
    'Miscellaneous', nil, 1, '', True);
  Demos.RegisterDemo('iOS', 'Creating an iOS application',
    'Demonstrates development of an application working with SQLite database ' +
    'for iOS in Delphi XE2 using LiteDAC.',
    'Miscellaneous', nil, 1, '', True);
{$ENDIF}

{$IFNDEF CLR}
  Demos.RegisterCategory('', '', -1, True);
{$ENDIF}
{$IFNDEF CLR}
  Demos.RegisterCategory('ThirdParty', '', -1, True);
  Demos.RegisterDemo('FastReport', 'FastReport hint',
    'Demonstrates how LiteDAC can be used with FastReport components. ' +
    'This project consists of two parts. The first part is several packages ' +
    'that integrate LiteDAC components into the FastReport editor. ' +
    'The second part is a demo application that lets you design and preview ' +
    'reports with LiteDAC technology in the FastReport editor.',
    'ThirdParty', nil, 1, '', True);
{$ENDIF}

end;

procedure TLiteDacForm.lbAboutClick(Sender: TObject);
begin
  inherited;

  ShowAbout;
end;

procedure TLiteDacForm.cbDebugClick(Sender: TObject);
begin
  inherited;
  scCreate.Debug := (Sender as TCheckBox).Checked;
  scDrop.Debug := (Sender as TCheckBox).Checked;
end;

procedure TLiteDacForm.InitializeDemoFrame(Frame: TDemoFrame;
  DemoType: TDemoType);
begin
  inherited;
end;

procedure TLiteDacForm.cbDirectClick(Sender: TObject);
begin
  LiteConnection.Options.Direct := cbDirect.Checked;
end;

procedure TLiteDacForm.FormResize(Sender: TObject);
begin
  inherited;

  cbDirect.Left := cbDebug.Left - (cbDirect.Width + 10);
  pnOpenDemoDir.Left := cbDirect.Left - (pnOpenDemoDir.Width + 15);
  pnShowSource.Left := pnOpenDemoDir.Left - (pnShowSource.Width + 15);
end;

procedure TLiteDacForm.sbConnectClick(Sender: TObject);
begin
  inherited;
end;

{$IFDEF FPC}
initialization
  {$i LiteDacDemoForm.lrs}
{$ENDIF}

end.

